
/*  This file is part of the program psim.

    Copyright (C) 1994-1995, Andrew Cagney <cagney@highland.com.au>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    --

    This file was generated by the program igen */

#ifndef _ICACHE_C_
#define _ICACHE_C_


#include "cpu.h"
#include "idecode.h"
#include "semantics.h"
#include "icache.h"
#ifdef HAVE_COMMON_FPU
#include "sim-inline.h"
#include "sim-fpu.h"
#endif
#include "support.h"


PSIM_INLINE_ICACHE\
(unsigned_word)
icache_illegal
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 983 "./ppc-instructions"
  program_interrupt(processor, cia,
                    illegal_instruction_program_interrupt);
  error("Internal function must longjump\n");
  return 0;
#line 54 "icache.c"
}

PSIM_INLINE_ICACHE\
(unsigned_word)
icache_floating_point_unavailable
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 991 "./ppc-instructions"
  floating_point_unavailable_interrupt(processor, cia);
  error("Internal function must longjump\n");
  return 0;
#line 70 "icache.c"
}


EXTERN_ICACHE\
(unsigned_word)
icache_instruction_call(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_instruction_call
#undef MY_PREFIX
#define MY_PREFIX instruction_call
#line 1494 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode instruction_call\n"));
#line 87 "icache.c"

  /* validate: 0.0,6./,11./,16./,21./,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x1)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #define get-values-from-icache|put-values-in-icache */
#line 96 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_instruction_call;

    /* semantic routine */
#line 1494 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics instruction_call\n"));
#line 106 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_instruction_call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1494 "./ppc-instructions"
    {
      if (!os_emul_instruction_call(processor, cia, real_addr(cia, 1)))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 121 "icache.c"
    return nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #undef get-values-from-icache|put-values-in-icache */
#line 124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word_Immediate(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Trap_Word_Immediate
#line 2898 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word Immediate\n"));
#line 143 "icache.c"

  /* validate: 0.3,6.TO,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 2898 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.D.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 163 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word_Immediate;

    /* semantic routine */
#line 2898 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word Immediate\n"));
#line 173 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2898 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTS(SI);
      if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 195 "icache.c"
    return nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 199 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Immediate(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Immediate
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Immediate
#line 2688 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Immediate\n"));
#line 218 "icache.c"

  /* validate: 0.7,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x1c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 242 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Immediate;

    /* semantic routine */
#line 2688 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Immediate\n"));
#line 252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2688 "./ppc-instructions"
    {
      signed_word prod = *rA * EXTS(SI);
      *rT = prod;
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 267 "icache.c"
    return nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Immediate_Carrying(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Immediate_Carrying
#line 2575 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Immediate Carrying\n"));
#line 290 "icache.c"

  /* validate: 0.8,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x20000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 314 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Immediate_Carrying;

    /* semantic routine */
#line 2575 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Immediate Carrying\n"));
#line 324 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2575 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(EXTS(SI));
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 342 "icache.c"
    return nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 346 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical_Immediate(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical_Immediate
#line 2834 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical Immediate\n"));
#line 365 "icache.c"

  /* validate: 0.10,6.BF,9./,10.L,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x28000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2834 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2834 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 389 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical_Immediate;

    /* semantic routine */
#line 2834 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical Immediate\n"));
#line 399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2834 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b = UI;
        if (L == 0)
          a = MASKED(*rA, 32, 63);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 424 "icache.c"
    return nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 428 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Immediate(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Immediate
#line 2792 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Immediate\n"));
#line 447 "icache.c"

  /* validate: 0.11,6.BF,9./,10.L,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x2c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2792 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 471 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Immediate;

    /* semantic routine */
#line 2792 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Immediate\n"));
#line 481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2792 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b = EXTS(SI);
        if (L == 0)
          a = EXTENDED(*rA);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 506 "icache.c"
    return nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 510 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying
#line 2555 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying\n"));
#line 529 "icache.c"

  /* validate: 0.12,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x30000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 553 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying;

    /* semantic routine */
#line 2555 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying\n"));
#line 563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2555 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 579 "icache.c"
    return nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying_and_Record(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying_and_Record
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying_and_Record
#line 2565 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying and Record\n"));
#line 602 "icache.c"

  /* validate: 0.13,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x34000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 626 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying_and_Record;

    /* semantic routine */
#line 2565 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying and Record\n"));
#line 636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying_and_Record, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2565 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 1/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 1/*Rc*/);
    }
#line 652 "icache.c"
    return nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 656 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA0(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA0
#line 2513 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 675 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 699 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA0;

    /* semantic routine */
#line 2513 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2513 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 725 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA
#line 2513 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 748 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 772 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA;

    /* semantic routine */
#line 2513 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 782 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2513 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 798 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA0
#line 2523 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 821 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 845 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA0;

    /* semantic routine */
#line 2523 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 855 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2523 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 871 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 875 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA
#line 2523 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 894 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 918 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA;

    /* semantic routine */
#line 2523 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 928 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2523 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 944 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 967 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 987 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1046 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1050 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1069 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1089 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1099 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1148 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1171 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1191 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1201 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1250 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1254 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1273 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1293 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1303 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1352 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1356 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1375 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1395 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1405 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1454 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1458 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1477 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1497 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1507 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1556 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1560 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1579 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1599 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1658 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1681 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1701 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1760 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1783 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1803 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1862 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1866 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1885 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1905 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1915 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1964 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1987 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2007 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2066 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2070 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2089 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2109 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2119 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2168 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2172 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2191 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2211 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2270 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2274 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2293 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2313 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2372 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2376 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2395 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2415 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2425 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2474 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2478 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2497 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2517 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2527 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2576 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2599 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2619 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2629 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2678 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2701 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2721 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2731 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2780 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2784 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2803 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2823 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2882 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2886 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2905 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2925 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2984 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2988 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3007 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3027 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3037 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3086 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3090 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3109 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3129 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3139 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3188 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3211 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3231 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3290 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3294 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3313 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3333 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3343 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3392 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3396 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3415 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3435 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3494 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3498 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3517 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3537 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3547 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3596 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3600 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3619 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3639 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3649 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3698 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3702 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3721 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3741 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3800 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3804 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3823 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3843 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3902 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3906 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3925 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3945 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3955 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4004 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4008 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4027 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4047 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4057 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4106 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4110 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4129 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4149 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4159 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4208 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4212 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4231 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4251 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4261 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4310 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4314 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4333 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4353 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4412 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4416 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4435 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4455 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4514 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4518 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4537 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4557 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4616 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4620 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4639 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4659 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4718 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4722 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4741 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4761 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4820 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4824 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4843 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4863 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4873 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4922 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4926 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4945 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4965 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5024 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5028 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5047 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5067 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5077 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5126 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5130 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5149 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5169 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5179 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5228 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5232 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5251 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5271 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5330 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5334 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5353 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5373 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5383 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5432 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5436 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5455 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5475 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5534 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5538 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5557 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5577 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5587 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5636 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5659 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5679 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5738 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5742 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5761 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5781 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5840 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5844 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5863 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5883 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5942 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5946 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5965 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5985 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5995 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6044 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6067 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6087 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6146 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6150 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6169 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6189 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6199 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6248 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6252 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6271 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6291 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6350 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6354 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6373 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6393 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6452 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6475 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6495 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6554 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6577 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6597 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6656 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6679 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6699 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6758 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6762 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6781 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6801 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6860 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6883 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6903 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6962 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6966 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6985 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7005 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7064 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7068 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7087 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7107 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7166 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7170 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7189 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7209 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7268 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7272 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7291 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7311 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7370 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7374 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7393 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7413 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7472 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7476 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7495 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7515 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7525 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7574 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7578 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7597 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7617 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7627 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7676 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7680 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7699 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7719 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7778 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7782 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7801 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7821 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7831 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7880 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7884 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7903 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7923 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7982 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7986 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8005 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8025 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8084 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8107 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8127 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8186 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8190 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8209 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8229 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8288 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8292 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8311 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8331 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8390 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8394 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8413 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8433 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8443 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8492 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8496 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8515 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8535 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8594 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8617 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8637 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8647 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8696 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8719 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8739 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8749 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8798 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8821 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8841 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8900 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8923 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8943 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9002 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9006 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9025 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9045 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9104 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9127 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9147 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9206 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9210 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9229 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9249 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9259 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9308 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9331 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9351 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9361 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9410 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9414 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9433 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9453 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9463 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9512 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9535 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9555 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9614 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9618 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9637 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9657 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9667 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9716 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9720 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9739 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9759 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9818 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9841 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9861 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9871 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9920 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9943 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9963 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9973 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10022 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10026 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10045 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10065 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10075 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10124 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10147 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10167 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10177 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10226 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10230 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10249 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10269 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10279 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10328 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10332 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10351 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10371 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10430 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10434 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10453 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10473 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10532 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10536 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10555 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10575 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10634 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10638 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10657 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10677 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10687 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10736 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10759 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10779 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10789 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10838 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10842 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10861 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10881 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10891 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10940 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10944 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10963 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10983 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11042 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11046 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11065 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11085 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11144 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11167 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11187 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11246 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11269 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11289 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11299 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11348 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11371 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11391 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11401 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11450 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11454 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11473 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11493 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11503 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11552 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11556 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11575 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11595 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11654 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11658 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11677 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11697 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11707 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11756 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11779 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11799 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11858 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11862 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11881 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11901 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11911 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11960 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11964 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11983 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12003 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12013 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12062 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12066 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12085 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12105 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12164 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12168 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12187 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12207 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12266 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12289 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12309 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12319 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12368 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12391 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12411 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12470 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12474 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12493 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12513 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12523 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12572 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12576 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12595 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12615 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12674 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12678 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12697 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12717 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12776 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12780 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12799 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12819 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12878 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12882 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12901 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12921 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12931 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12980 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12984 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13003 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13023 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13082 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13086 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13105 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13125 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13184 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13188 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13207 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13227 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13237 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13286 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13290 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13309 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13329 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13388 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13392 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13411 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13431 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13490 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13494 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13513 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13533 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13543 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13592 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13596 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13615 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13635 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13694 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13698 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13717 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13737 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13747 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13796 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13819 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13839 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13849 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13898 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13902 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13921 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13941 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13951 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 14000 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14004 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_System_Call(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_System_Call
#undef MY_PREFIX
#define MY_PREFIX System_Call
#line 1631 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode System Call\n"));
#line 14023 "icache.c"

  /* validate: 0.17,6./,11./,16./,30.1,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x44000002)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #define get-values-from-icache|put-values-in-icache */
#line 14032 "icache.c"

#define insn cache_entry->crack.SC.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_System_Call;

    /* semantic routine */
#line 1631 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics System Call\n"));
#line 14042 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_System_Call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1631 "./ppc-instructions"
    {
      if (CURRENT_MODEL_ISSUE > 0)
        model_serialize(MY_INDEX, cpu_model(processor));
      system_call_interrupt(processor, cia);
    }
#line 14057 "icache.c"
    return nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14060 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK0
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14079 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14094 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK0;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14104 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14123 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14126 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK1
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14145 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14160 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK1;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14170 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14189 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK0
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14211 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14226 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK0;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14236 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14255 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14258 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK1
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14277 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14292 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK1;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14302 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14321 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14324 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_Condition_Register_Field(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_Condition_Register_Field
#undef MY_PREFIX
#define MY_PREFIX Move_Condition_Register_Field
#line 1710 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move Condition Register Field\n"));
#line 14343 "icache.c"

  /* validate: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0x4c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.XL.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.XL.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 1710 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.XL.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 14358 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_Condition_Register_Field;

    /* semantic routine */
#line 1710 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move Condition Register Field\n"));
#line 14368 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_Condition_Register_Field, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1710 "./ppc-instructions"
    {
      MBLIT32(CR, 4*BF, 4*BF+3, EXTRACTED32(CR, 4*BFA, 4*BFA+3));
      PPC_INSN_CR(BF_BITMASK, 1 << BFA);
    }
#line 14382 "icache.c"
    return nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK0
#line 1562 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14404 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1562 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1562 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14420 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK0;

    /* semantic routine */
#line 1562 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14430 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1562 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14471 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14475 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK1
#line 1562 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14494 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1562 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1562 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14510 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK1;

    /* semantic routine */
#line 1562 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14520 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1562 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14561 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NOR
#line 1675 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NOR\n"));
#line 14584 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.33,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000042)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14607 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NOR;

    /* semantic routine */
#line 1675 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NOR\n"));
#line 14617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1675 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ || CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14631 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14636 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Return_From_Interrupt(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Return_From_Interrupt
#undef MY_PREFIX
#define MY_PREFIX Return_From_Interrupt
#line 4863 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Return From Interrupt\n"));
#line 14655 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.50,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c000064)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #define get-values-from-icache|put-values-in-icache */
#line 14664 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Return_From_Interrupt;

    /* semantic routine */
#line 4863 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Return From Interrupt\n"));
#line 14674 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Return_From_Interrupt, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4863 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor)) {
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      }
      else {
        MSR = (MASKED(SRR1, 0, 32)
               | MASKED(SRR1, 37, 41)
               | MASKED(SRR1, 48, 63));
        NIA = MASKED(SRR0, 0, 61);
        cpu_synchronize_context(processor, cia);
        check_masked_interrupts(processor);
      }
    }
#line 14698 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND_with_Complement(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND_with_Complement
#line 1691 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND with Complement\n"));
#line 14720 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.129,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000102)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14743 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND_with_Complement;

    /* semantic routine */
#line 1691 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND with Complement\n"));
#line 14753 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1691 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14767 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Synchronize(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Instruction_Synchronize
#line 4776 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Synchronize\n"));
#line 14791 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.150,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c00012c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #define get-values-from-icache|put-values-in-icache */
#line 14800 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Synchronize;

    /* semantic routine */
#line 4776 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Synchronize\n"));
#line 14810 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4776 "./ppc-instructions"
    {
      cpu_synchronize_context(processor, cia);
      PPC_INSN_INT(0, 0, 0);
    }
#line 14824 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14827 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_XOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_XOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_XOR
#line 1659 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register XOR\n"));
#line 14846 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.193,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000182)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14869 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_XOR;

    /* semantic routine */
#line 1659 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register XOR\n"));
#line 14879 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1659 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ != CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14893 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14898 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NAND(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NAND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NAND
#line 1667 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NAND\n"));
#line 14917 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.225,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c0001c2)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14940 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NAND;

    /* semantic routine */
#line 1667 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NAND\n"));
#line 14950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1667 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ && CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14964 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14969 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND
#line 1643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND\n"));
#line 14988 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.257,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000202)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15011 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND;

    /* semantic routine */
#line 1643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND\n"));
#line 15021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1643 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15035 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_Equivalent(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_Equivalent
#line 1683 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register Equivalent\n"));
#line 15059 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.289,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000242)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15082 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_Equivalent;

    /* semantic routine */
#line 1683 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register Equivalent\n"));
#line 15092 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1683 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ == CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15106 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15111 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR_with_Complement(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR_with_Complement
#line 1699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR with Complement\n"));
#line 15130 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.417,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000342)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15153 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR_with_Complement;

    /* semantic routine */
#line 1699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR with Complement\n"));
#line 15163 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1699 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15177 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15182 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR
#line 1651 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR\n"));
#line 15201 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.449,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000382)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15224 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR;

    /* semantic routine */
#line 1651 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR\n"));
#line 15234 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1651 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15248 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK0
#line 1597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15272 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1597 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1597 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 15288 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK0;

    /* semantic routine */
#line 1597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1597 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15335 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK1
#line 1597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15358 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1597 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1597 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 15374 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK1;

    /* semantic routine */
#line 1597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15384 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1597 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15421 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0
#line 3232 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15444 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3232 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15474 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0;

    /* semantic routine */
#line 3232 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15484 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3232 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15506 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15510 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1
#line 3232 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15529 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3232 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15559 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1;

    /* semantic routine */
#line 3232 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15569 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3232 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15591 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15595 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0
#line 3179 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15614 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3179 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15644 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3179 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15654 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3179 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15678 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1
#line 3179 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15701 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3179 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15731 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3179 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3179 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15765 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc0
#line 3211 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15788 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15822 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3211 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3211 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15850 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15854 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc1
#line 3211 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15873 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15907 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3211 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15917 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3211 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15935 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15939 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate(cpu *processor,
                    instruction_word instruction,
                    unsigned_word cia,
                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate
#line 2975 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate\n"));
#line 15958 "icache.c"

  /* validate: 0.24,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x60000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2975 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 15982 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate;

    /* semantic routine */
#line 2975 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate\n"));
#line 15992 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2975 "./ppc-instructions"
    {
      *rA = *rS | UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16007 "icache.c"
    return nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate_Shifted(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate_Shifted
#line 2984 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate Shifted\n"));
#line 16030 "icache.c"

  /* validate: 0.25,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x64000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2984 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16054 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate_Shifted;

    /* semantic routine */
#line 2984 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate Shifted\n"));
#line 16064 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2984 "./ppc-instructions"
    {
      *rA = *rS | (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16079 "icache.c"
    return nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate
#line 2993 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate\n"));
#line 16102 "icache.c"

  /* validate: 0.26,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x68000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2993 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16126 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate;

    /* semantic routine */
#line 2993 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate\n"));
#line 16136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2993 "./ppc-instructions"
    {
      *rA = *rS ^ UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16151 "icache.c"
    return nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16155 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate_Shifted
#line 3002 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate Shifted\n"));
#line 16174 "icache.c"

  /* validate: 0.27,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x6c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3002 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16198 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate_Shifted;

    /* semantic routine */
#line 3002 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate Shifted\n"));
#line 16208 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3002 "./ppc-instructions"
    {
      *rA = *rS ^ (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16223 "icache.c"
    return nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16227 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate
#line 2955 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate\n"));
#line 16246 "icache.c"

  /* validate: 0.28,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x70000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2955 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16270 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate;

    /* semantic routine */
#line 2955 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate\n"));
#line 16280 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2955 "./ppc-instructions"
    {
      *rA = *rS & UI;
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16296 "icache.c"
    return nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16300 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate_Shifted
#line 2965 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate Shifted\n"));
#line 16319 "icache.c"

  /* validate: 0.29,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x74000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2965 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16343 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate_Shifted;

    /* semantic routine */
#line 2965 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate Shifted\n"));
#line 16353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2965 "./ppc-instructions"
    {
      *rA = *rS & (UI << 16);
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16369 "icache.c"
    return nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare
#undef MY_PREFIX
#define MY_PREFIX Compare
#line 2811 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare\n"));
#line 16392 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2811 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16420 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare;

    /* semantic routine */
#line 2811 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare\n"));
#line 16430 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2811 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b;
        if (L == 0) {
          a = EXTENDED(*rA);
          b = EXTENDED(*rB);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 16459 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16463 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word(cpu *processor,
                 instruction_word instruction,
                 unsigned_word cia,
                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word
#undef MY_PREFIX
#define MY_PREFIX Trap_Word
#line 2931 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word\n"));
#line 16482 "icache.c"

  /* validate: 0.31,6.TO,11.RA,16.RB,21.4,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000008)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #define get-values-from-icache|put-values-in-icache */
#line 2931 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.X.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16506 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word;

    /* semantic routine */
#line 2931 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word\n"));
#line 16516 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2931 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTENDED(*rB);
      if (TO == 12 && rA == rB) {
        ITRACE(trace_breakpoint, ("breakpoint\n"));
        cpu_halt(processor, cia, was_trap, 0);
      }
      else if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 16542 "icache.c"
    return nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16546 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc0
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16565 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16597 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16626 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16630 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc1
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16649 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16681 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16691 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16710 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16714 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc0
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16733 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16765 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16775 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16791 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16795 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc1
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16814 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16846 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16856 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16872 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16876 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc0
#line 2731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16895 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2731 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16925 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc0;

    /* semantic routine */
#line 2731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 16935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2731 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16954 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16958 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc1
#line 2731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16977 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2731 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17007 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc1;

    /* semantic routine */
#line 2731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 17017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2731 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17036 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Condition_Register(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Condition_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Condition_Register
#line 3454 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Condition Register\n"));
#line 17059 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.19,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000026)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 17074 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Condition_Register;

    /* semantic routine */
#line 3454 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Condition Register\n"));
#line 17084 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Condition_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3454 "./ppc-instructions"
    {
      *rT = (unsigned32)CR;
      PPC_INSN_MFCR(RT_BITMASK);
    }
#line 17098 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #undef get-values-from-icache|put-values-in-icache */
#line 17101 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_And_Reserve_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_And_Reserve_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_And_Reserve_Indexed
#line 2423 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word And Reserve Indexed\n"));
#line 17120 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.20,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17148 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_And_Reserve_Indexed;

    /* semantic routine */
#line 2423 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word And Reserve Indexed\n"));
#line 17158 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_And_Reserve_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2423 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      RESERVE = 1;
      RESERVE_ADDR = real_addr(EA, 1/*is-read?*/);
      RESERVE_DATA = MEM(unsigned, EA, 4);
      *rT = RESERVE_DATA;
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17180 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_Indexed
#line 1899 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero Indexed\n"));
#line 17203 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.23,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17231 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_Indexed;

    /* semantic routine */
#line 1899 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero Indexed\n"));
#line 17241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1899 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17260 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17264 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc0
#line 3251 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17283 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3251 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17313 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc0;

    /* semantic routine */
#line 3251 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3251 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17348 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc1
#line 3251 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17371 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3251 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17401 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc1;

    /* semantic routine */
#line 3251 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17411 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3251 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17436 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc0
#line 3130 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17459 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3130 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17483 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc0;

    /* semantic routine */
#line 3130 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17493 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3130 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17515 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17519 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc1
#line 3130 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17538 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3130 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17562 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc1;

    /* semantic routine */
#line 3130 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17572 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3130 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc0
#line 3011 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17617 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3011 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17647 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc0;

    /* semantic routine */
#line 3011 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17657 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3011 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17673 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc1
#line 3011 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17696 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3011 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17726 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc1;

    /* semantic routine */
#line 3011 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17736 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3011 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17752 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17756 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical
#line 2853 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical\n"));
#line 17775 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2853 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17803 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical;

    /* semantic routine */
#line 2853 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical\n"));
#line 17813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2853 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b;
        if (L == 0) {
          a = MASKED(*rA, 32, 63);
          b = MASKED(*rB, 32, 63);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 17842 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17846 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc0
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17865 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17897 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc0;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17907 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17925 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17929 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc1
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17948 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17980 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc1;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18008 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18012 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Store(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Store
#line 4813 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Store\n"));
#line 18031 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.54,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00006c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18053 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Store;

    /* semantic routine */
#line 4813 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Store\n"));
#line 18063 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4813 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18077 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update_Indexed
#line 1926 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update Indexed\n"));
#line 18100 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.55,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00006e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18128 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1926 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update Indexed\n"));
#line 18138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1926 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18158 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18162 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc0
#line 3071 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18181 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3071 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18211 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc0;

    /* semantic routine */
#line 3071 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3071 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18237 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18241 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc1
#line 3071 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18260 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3071 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18290 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc1;

    /* semantic routine */
#line 3071 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18300 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3071 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18316 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18320 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc0
#line 2716 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18339 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2716 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18369 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc0;

    /* semantic routine */
#line 2716 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2716 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18398 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18402 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc1
#line 2716 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18421 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2716 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18451 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc1;

    /* semantic routine */
#line 2716 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18461 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2716 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18480 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18484 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Machine_State_Register(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Machine_State_Register
#line 4900 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Machine State Register\n"));
#line 18503 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.83,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c0000a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 18518 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Machine_State_Register;

    /* semantic routine */
#line 4900 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Machine State Register\n"));
#line 18528 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4900 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        *rT = MSR;
        check_masked_interrupts(processor);
      }
    }
#line 18547 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #undef get-values-from-icache|put-values-in-icache */
#line 18550 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Flush(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Flush
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Flush
#line 4821 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Flush\n"));
#line 18569 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.86,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0000ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18591 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Flush;

    /* semantic routine */
#line 4821 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Flush\n"));
#line 18601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Flush, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4821 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Flush\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18615 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18619 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_Indexed
#line 1737 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero Indexed\n"));
#line 18638 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.87,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18666 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_Indexed;

    /* semantic routine */
#line 1737 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero Indexed\n"));
#line 18676 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1737 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 18695 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18699 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc0
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18718 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18744 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc0;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18754 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18771 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18775 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc1
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18794 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18820 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc1;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18830 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18847 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18851 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update_Indexed
#line 1764 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update Indexed\n"));
#line 18870 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.119,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18898 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1764 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update Indexed\n"));
#line 18908 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1764 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18928 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc0
#line 3051 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 18951 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3051 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18981 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc0;

    /* semantic routine */
#line 3051 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 18991 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3051 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19007 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc1
#line 3051 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 19030 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3051 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19060 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc1;

    /* semantic routine */
#line 3051 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 19070 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3051 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19086 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19090 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc0
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19109 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19141 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc0;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19151 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19169 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19173 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc1
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19192 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19224 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc1;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19234 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19252 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc0
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19275 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19307 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc0;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc1
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19357 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19389 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc1;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19416 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_Fields(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_Fields
#line 3431 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register Fields\n"));
#line 19439 "icache.c"

  /* validate: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc100fff) != 0x7c000120)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3431 "./ppc-instructions"
    unsigned const FXM UNUSED = cache_entry->crack.XFX.FXM = EXTRACTED32(instruction, 12, 19); /* default in icache - extraction */
#line 19456 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_Fields;

    /* semantic routine */
#line 3431 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register Fields\n"));
#line 19466 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3431 "./ppc-instructions"
    {
      if (FXM == 0xff) {
        CR = *rS;
      }
      else {
        unsigned_word mask = 0;
        unsigned_word f;
        for (f = 0; f < 8; f++) {
          if (FXM & (0x80 >> f))
            mask |= (0xf << 4*(7-f));
        }
        CR = (MASKED(*rS, 32, 63) & mask) | (CR & ~mask);
      }
      PPC_INSN_MTCR(RS_BITMASK, FXM);
    }
#line 19491 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19494 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Machine_State_Register(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Machine_State_Register
#line 4887 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Machine State Register\n"));
#line 19513 "icache.c"

  /* validate: 0.31,6.RS,11./,16./,21.146,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000124)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 19528 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Machine_State_Register;

    /* semantic routine */
#line 4887 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Machine State Register\n"));
#line 19538 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4887 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        MSR = *rS;
        check_masked_interrupts(processor);
      }
    }
#line 19557 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19560 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Conditional_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Conditional_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Conditional_Indexed
#line 2451 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Conditional Indexed\n"));
#line 19579 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.150,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012d)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19607 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Conditional_Indexed;

    /* semantic routine */
#line 2451 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Conditional Indexed\n"));
#line 19617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Conditional_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2451 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (RESERVE) {
        if (RESERVE_ADDR == real_addr(EA, 0/*is-read?*/)
            && /*HACK*/ RESERVE_DATA == MEM(unsigned, EA, 4)) {
          STORE(EA, 4, *rS);
          CR_SET_XER_SO(0, cr_i_zero);
        }
        else {
          /* ment to randomly to store, we never do! */	
          CR_SET_XER_SO(0, 0);
        }
        RESERVE = 0;
      }
      else {
        CR_SET_XER_SO(0, 0);
      }
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 1/*Rc*/);
    }
#line 19650 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19654 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Indexed
#line 2126 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Indexed\n"));
#line 19673 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.151,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19701 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Indexed;

    /* semantic routine */
#line 2126 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Indexed\n"));
#line 19711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2126 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19730 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19734 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update_Indexed
#line 2153 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update Indexed\n"));
#line 19753 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.183,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00016e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19781 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update_Indexed;

    /* semantic routine */
#line 2153 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update Indexed\n"));
#line 19791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2153 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19811 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19815 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc0
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19834 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19860 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19870 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19887 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19891 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc1
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19910 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19936 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19946 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19963 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19967 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc0
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 19986 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20012 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20022 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20038 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc1
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 20061 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20087 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20113 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register
#line 4933 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register\n"));
#line 20136 "icache.c"

  /* validate: 0.31,6.RS,11./,12.SR,16./,21.210,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0001a4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 4933 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 20153 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register;

    /* semantic routine */
#line 4933 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register\n"));
#line 20163 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4933 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(SR) = *rS;
    }
#line 20180 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20183 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_Indexed
#line 2018 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte Indexed\n"));
#line 20202 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.215,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20230 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_Indexed;

    /* semantic routine */
#line 2018 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte Indexed\n"));
#line 20240 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2018 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20259 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20263 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc0
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20282 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20308 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20318 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20336 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20340 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc1
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20359 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20385 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20395 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20413 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc0
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20436 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20462 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20472 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20489 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20493 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc1
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20512 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20538 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20548 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20565 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20569 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc0
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20588 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20620 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc0;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20630 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20651 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc1
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20674 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20706 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc1;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20716 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20737 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register_Indirect(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register_Indirect
#line 4944 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register Indirect\n"));
#line 20760 "icache.c"

  /* validate: 0.31,6.RS,11./,16.RB,21.242,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c0001e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20781 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register_Indirect;

    /* semantic routine */
#line 4944 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register Indirect\n"));
#line 20791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4944 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(EXTRACTED32(*rB, 0, 3)) = *rS;
    }
#line 20808 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20811 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch_for_Store(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch_for_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch_for_Store
#line 4797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch for Store\n"));
#line 20830 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.246,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0001ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20852 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch_for_Store;

    /* semantic routine */
#line 4797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch for Store\n"));
#line 20862 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch_for_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4797 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch for Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 20876 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20880 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update_Indexed
#line 2045 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update Indexed\n"));
#line 20899 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.247,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20927 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update_Indexed;

    /* semantic routine */
#line 2045 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update Indexed\n"));
#line 20937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2045 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20957 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc0
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 20980 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21012 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc0;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21022 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21038 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc1
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 21061 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21093 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc1;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21103 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21119 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21123 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch
#line 4789 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch\n"));
#line 21142 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.278,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00022c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21164 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch;

    /* semantic routine */
#line 4789 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch\n"));
#line 21174 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4789 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 21188 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_Indexed
#line 1791 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero Indexed\n"));
#line 21211 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.279,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00022e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21239 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_Indexed;

    /* semantic routine */
#line 1791 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero Indexed\n"));
#line 21249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1791 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 21268 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21272 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc0
#line 3061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21291 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21321 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc0;

    /* semantic routine */
#line 3061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21331 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3061 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21347 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21351 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc1
#line 3061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21370 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21400 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc1;

    /* semantic routine */
#line 3061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21410 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3061 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21426 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21430 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_Entry(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_Entry
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_Entry
#line 4982 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate Entry\n"));
#line 21449 "icache.c"

  /* validate: 0.31,6./,11./,16.RB,21.306,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffff07ff) != 0x7c000264)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #define get-values-from-icache|put-values-in-icache */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21464 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_Entry;

    /* semantic routine */
#line 4982 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate Entry\n"));
#line 21474 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_Entry, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4982 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_entry(proc, *rB);
          nr++;
        }
      }
    }
#line 21499 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update_Indexed(cpu *processor,
                                                  instruction_word instruction,
                                                  unsigned_word cia,
                                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update_Indexed
#line 1818 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update Indexed\n"));
#line 21521 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.311,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00026e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21549 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1818 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update Indexed\n"));
#line 21559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1818 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 21579 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc0
#line 3031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21602 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3031 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21632 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc0;

    /* semantic routine */
#line 3031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21642 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3031 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21658 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc1
#line 3031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21681 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3031 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21711 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc1;

    /* semantic routine */
#line 3031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21721 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3031 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21737 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR256(cpu *processor,
                                                 instruction_word instruction,
                                                 unsigned_word cia,
                                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR256
#line 3390 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21760 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3390 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 21777 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3390 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21787 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3390 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21834 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR
#line 3390 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21856 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3390 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 21873 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3390 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21883 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3390 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21930 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_Indexed
#line 1845 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic Indexed\n"));
#line 21952 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.343,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21980 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_Indexed;

    /* semantic routine */
#line 1845 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic Indexed\n"));
#line 21990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1845 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 22009 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22013 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_All(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_All
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_All
#line 4997 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate All\n"));
#line 22032 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.370,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0002e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #define get-values-from-icache|put-values-in-icache */
#line 22041 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_All;

    /* semantic routine */
#line 4997 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate All\n"));
#line 22051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_All, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4997 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_all(proc);
          nr++;
        }
      }
    }
#line 22076 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22079 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Time_Base(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Time_Base
#undef MY_PREFIX
#define MY_PREFIX Move_From_Time_Base
#line 4840 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Time Base\n"));
#line 22098 "icache.c"

  /* validate: 0.31,6.RT,11.tbr,21.371,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002e6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4840 "./ppc-instructions"
    unsigned const tbr UNUSED = cache_entry->crack.XFX.tbr = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 22115 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Time_Base;

    /* semantic routine */
#line 4840 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Time Base\n"));
#line 22125 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Time_Base, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4840 "./ppc-instructions"
    {
      int n = (tbr_5_9_ << 5) | tbr_0_4_;
      if (n == 268) {
        if (is_64bit_implementation) *rT = TB;
        else                         *rT = EXTRACTED64(TB, 32, 63);
      }
      else if (n == 269) {
        if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
        else                         *rT = EXTRACTED64(TB, 0, 31);
      }
      else
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 22149 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update_Indexed(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update_Indexed
#line 1872 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update Indexed\n"));
#line 22171 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.375,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22199 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update_Indexed;

    /* semantic routine */
#line 1872 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update Indexed\n"));
#line 22209 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1872 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 22229 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22233 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_Indexed(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_Indexed
#line 2072 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Indexed\n"));
#line 22252 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.407,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00032e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22280 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_Indexed;

    /* semantic routine */
#line 2072 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Indexed\n"));
#line 22290 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2072 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22309 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc0
#line 3081 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22332 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3081 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22362 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc0;

    /* semantic routine */
#line 3081 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22372 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3081 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22388 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22392 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc1
#line 3081 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22411 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3081 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22441 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc1;

    /* semantic routine */
#line 3081 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22451 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3081 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22467 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22471 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update_Indexed(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update_Indexed
#line 2099 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update Indexed\n"));
#line 22490 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.439,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00036e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22518 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update_Indexed;

    /* semantic routine */
#line 2099 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update Indexed\n"));
#line 22528 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2099 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22548 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22552 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc0(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc0
#line 3021 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22571 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3021 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22601 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc0;

    /* semantic routine */
#line 3021 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22611 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3021 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22627 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22631 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc1(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc1
#line 3021 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22650 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3021 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22680 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc1;

    /* semantic routine */
#line 3021 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22690 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3021 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22706 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22710 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc0
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22729 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22761 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc0;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22796 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc1
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22819 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22851 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc1;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22886 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22890 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR256(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR256
#line 3346 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 22909 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3346 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 22926 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3346 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 22936 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3346 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 22986 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22989 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR
#line 3346 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 23008 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3346 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 23025 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3346 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 23035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3346 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 23085 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Invalidate(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Invalidate
#line 4918 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Invalidate\n"));
#line 23107 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.470,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0003ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23129 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Invalidate;

    /* semantic routine */
#line 4918 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Invalidate\n"));
#line 23139 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4918 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        TRACE(trace_tbd,("Data Cache Block Invalidate\n"));
    }
#line 23156 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc0(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc0
#line 3041 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23179 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3041 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23209 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc0;

    /* semantic routine */
#line 3041 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3041 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23235 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc1(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc1
#line 3041 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23258 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3041 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23288 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc1;

    /* semantic routine */
#line 3041 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3041 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23314 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23318 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc0
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23337 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23369 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc0;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23405 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23409 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc1
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23428 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23460 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc1;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23470 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23496 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_XER(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_XER
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_XER
#line 3446 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from XER\n"));
#line 23519 "icache.c"

  /* validate: 0.31,6.BF,9./,11./,16./,21.512,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7fffff) != 0x7c000400)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 23532 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_XER;

    /* semantic routine */
#line 3446 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from XER\n"));
#line 23542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_XER, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3446 "./ppc-instructions"
    error("%s:%d:0x%08lx:%s unimplemented\n",
          itable[MY_INDEX].file, itable[MY_INDEX].line_nr, (long)cia, itable[MY_INDEX].name);
#line 23554 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc0
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23576 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23608 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23618 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23637 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc1
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23660 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23692 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23702 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23721 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc0
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23744 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23776 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23802 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23806 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc1
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23825 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23857 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23867 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23883 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23887 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Indexed(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Indexed
#line 2329 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Indexed\n"));
#line 23906 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.533,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23934 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Indexed;

    /* semantic routine */
#line 2329 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Indexed\n"));
#line 23944 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2329 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      n = EXTRACTED32(XER, 25, 31);
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if (((RT + nr >= 32)
           ? ((RA >= RT || RA < (RT + nr) % 32)
              || (RB >= RT || RB < (RT + nr) % 32))
           : ((RA >= RT && RA < RT + nr)
              || (RB >= RT && RB < RT + nr)))
          || (RT == RA || RT == RB))
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 23990 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23994 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_ByteReverse_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_ByteReverse_Indexed
#line 2216 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word Byte-Reverse Indexed\n"));
#line 24013 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.534,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24041 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_ByteReverse_Indexed;

    /* semantic routine */
#line 2216 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word Byte-Reverse Indexed\n"));
#line 24051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2216 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_4(MEM(unsigned, EA, 4));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24070 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24074 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_Indexed
#line 3479 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single Indexed\n"));
#line 24093 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.535,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24128 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3479 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single Indexed\n"));
#line 24138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3479 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24157 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24162 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc0
#line 3272 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24181 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3272 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24211 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc0;

    /* semantic routine */
#line 3272 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3272 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24246 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc1
#line 3272 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24269 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3272 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24299 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc1;

    /* semantic routine */
#line 3272 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24309 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3272 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc0
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24357 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24389 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc0;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24417 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24421 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc1
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24440 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24472 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc1;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24500 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Synchronize(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Synchronize
#undef MY_PREFIX
#define MY_PREFIX TLB_Synchronize
#line 5012 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Synchronize\n"));
#line 24523 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.566,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c00046c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #define get-values-from-icache|put-values-in-icache */
#line 24532 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Synchronize;

    /* semantic routine */
#line 5012 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Synchronize\n"));
#line 24542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 5012 "./ppc-instructions"
    {
      /* nothing happens here - always in sync */
    }
#line 24555 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update_Indexed
#line 3506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update Indexed\n"));
#line 24577 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.567,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00046e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24612 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update Indexed\n"));
#line 24622 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3506 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24642 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24647 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register
#line 4955 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register\n"));
#line 24666 "icache.c"

  /* validate: 0.31,6.RT,11./,12.SR,16./,21.595,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0004a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4955 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 24683 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register;

    /* semantic routine */
#line 4955 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register\n"));
#line 24693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4955 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(SR);
    }
#line 24710 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24713 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Immediate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Immediate
#line 2297 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Immediate\n"));
#line 24732 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.NB,21.597,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2297 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 24756 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Immediate;

    /* semantic routine */
#line 2297 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Immediate\n"));
#line 24766 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2297 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if ((RT + nr >= 32)
          ? (RA >= RT || RA < (RT + nr) % 32)
          : (RA >= RT && RA < RT + nr))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 24808 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24812 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Synchronize(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Synchronize
#line 2501 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Synchronize\n"));
#line 24831 "icache.c"

  /* validate: 0.31,6./,9.L,11./,16./,21.598,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xff9fffff) != 0x7c0004ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #define get-values-from-icache|put-values-in-icache */
#line 2501 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 9, 10); /* default in icache - extraction */
#line 24842 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Synchronize;

    /* semantic routine */
#line 2501 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Synchronize\n"));
#line 24852 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2501 "./ppc-instructions"
    {
      /* do nothing */
    }
#line 24865 "icache.c"
    return nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24868 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_Indexed
#line 3533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double Indexed\n"));
#line 24887 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.599,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24922 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double Indexed\n"));
#line 24932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3533 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24951 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24956 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc0
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 24975 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25001 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc0;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25011 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25028 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25032 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc1
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 25051 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25077 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc1;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25087 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25104 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update_Indexed
#line 3560 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update Indexed\n"));
#line 25127 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.631,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25162 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3560 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update Indexed\n"));
#line 25172 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3560 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 25192 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 25197 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc0
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25216 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25248 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc0;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25258 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25276 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25280 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc1
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25299 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25331 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc1;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25359 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25363 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc0
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25382 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25414 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc0;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25441 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc1
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25464 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25496 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc1;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25523 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25527 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register_Indirect(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register_Indirect
#line 4966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register Indirect\n"));
#line 25546 "icache.c"

  /* validate: 0.31,6.RT,11./,16.RB,21.659,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c000526)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25567 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register_Indirect;

    /* semantic routine */
#line 4966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register Indirect\n"));
#line 25577 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4966 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(EXTRACTED32(*rB, 0, 3));
    }
#line 25594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #undef get-values-from-icache|put-values-in-icache */
#line 25597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Indexed(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Indexed
#line 2387 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Indexed\n"));
#line 25616 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.661,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25644 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Indexed;

    /* semantic routine */
#line 2387 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Indexed\n"));
#line 25654 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2387 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      n = EXTRACTED32(XER, 25, 31);
      r = RS - 1;
      i = 32;
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 25687 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25691 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_ByteReversed_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_ByteReversed_Indexed
#line 2242 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Byte-Reversed Indexed\n"));
#line 25710 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.662,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25738 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2242 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Byte-Reversed Indexed\n"));
#line 25748 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2242 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SWAP_4(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 25767 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25771 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_Indexed
#line 3592 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single Indexed\n"));
#line 25790 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.663,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25825 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3592 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single Indexed\n"));
#line 25835 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3592 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25854 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25859 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update_Indexed
#line 3619 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update Indexed\n"));
#line 25878 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.695,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00056e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25913 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3619 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update Indexed\n"));
#line 25923 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3619 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25943 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc0
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 25967 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25993 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26003 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26020 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc1
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 26043 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26069 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26079 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26096 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26100 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc0
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26119 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26145 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26171 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26175 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc1
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26194 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26220 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26230 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26246 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Immedate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Immedate
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Immedate
#line 2365 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Immedate\n"));
#line 26269 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.NB,21.725,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2365 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 26293 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Immedate;

    /* semantic routine */
#line 2365 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Immedate\n"));
#line 26303 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Immedate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2365 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RS - 1;
      i = 32;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 26335 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_Indexed
#line 3646 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double Indexed\n"));
#line 26358 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.727,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26393 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3646 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double Indexed\n"));
#line 26403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3646 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26422 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc0
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26446 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26472 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26500 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc1
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26523 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26549 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26577 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc0
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26600 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26626 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26653 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc1
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26676 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26702 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26712 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26729 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc0
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26752 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26784 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc0;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26815 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26819 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc1
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26838 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26870 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc1;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26880 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26901 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26905 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update_Indexed
#line 3685 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update Indexed\n"));
#line 26924 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.759,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26959 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3685 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update Indexed\n"));
#line 26969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3685 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26989 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26994 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc0
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27013 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27045 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc0;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27071 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27075 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc1
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27094 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27126 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc1;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27152 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27156 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_ByteReverse_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_ByteReverse_Indexed
#line 2203 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Byte-Reverse Indexed\n"));
#line 27175 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.790,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00062c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27203 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_ByteReverse_Indexed;

    /* semantic routine */
#line 2203 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Byte-Reverse Indexed\n"));
#line 27213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2203 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_2(MEM(unsigned, EA, 2));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 27232 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27236 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc0
#line 3315 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27255 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3315 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27285 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc0;

    /* semantic routine */
#line 3315 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27295 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3315 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27328 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27332 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc1
#line 3315 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27351 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3315 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27381 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc1;

    /* semantic routine */
#line 3315 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27391 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3315 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27424 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27428 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc0
#line 3293 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27447 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3293 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3293 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27473 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc0;

    /* semantic routine */
#line 3293 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3293 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27509 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc1
#line 3293 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27532 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3293 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3293 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27558 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc1;

    /* semantic routine */
#line 3293 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27568 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3293 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Enforce_Inorder_Execution_of_IO(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Enforce_Inorder_Execution_of_IO
#undef MY_PREFIX
#define MY_PREFIX Enforce_Inorder_Execution_of_IO
#line 4829 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Enforce In-order Execution of I/O\n"));
#line 27617 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.854,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0006ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #define get-values-from-icache|put-values-in-icache */
#line 27626 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Enforce_Inorder_Execution_of_IO;

    /* semantic routine */
#line 4829 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Enforce In-order Execution of I/O\n"));
#line 27636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Enforce_Inorder_Execution_of_IO, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4829 "./ppc-instructions"
    {
      /* Since this model has no instruction overlap
         this instruction need do nothing */
    }
#line 27650 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #undef get-values-from-icache|put-values-in-icache */
#line 27653 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_ByteReversed_Indexed(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_ByteReversed_Indexed
#line 2229 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Byte-Reversed Indexed\n"));
#line 27672 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.918,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00072c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27700 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2229 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Byte-Reversed Indexed\n"));
#line 27710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2229 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, SWAP_2(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 27729 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc0
#line 3101 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27752 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3101 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27776 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc0;

    /* semantic routine */
#line 3101 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3101 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27802 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27806 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc1
#line 3101 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27825 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3101 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27849 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc1;

    /* semantic routine */
#line 3101 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27859 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3101 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27875 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27879 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc0
#line 3091 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27898 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3091 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27922 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc0;

    /* semantic routine */
#line 3091 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 27932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3091 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27948 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27952 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc1
#line 3091 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27971 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3091 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27995 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc1;

    /* semantic routine */
#line 3091 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 28005 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3091 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 28021 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc0
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28044 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28076 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc0;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28111 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28115 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc1
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28134 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28166 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc1;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28176 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28201 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Cache_Block_Invalidate(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Instruction_Cache_Block_Invalidate
#line 4765 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Cache Block Invalidate\n"));
#line 28224 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.982,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28246 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Cache_Block_Invalidate;

    /* semantic routine */
#line 4765 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Cache Block Invalidate\n"));
#line 28256 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4765 "./ppc-instructions"
    {
      /* blindly flush all instruction cache entries */
#if WITH_IDECODE_CACHE_SIZE
      cpu_flush_icache(processor);
#endif
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 28273 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Integer_Word_Indexed(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Integer_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Integer_Word_Indexed
#line 3658 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Integer Word Indexed\n"));
#line 28296 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.983,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0007ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28331 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Integer_Word_Indexed;

    /* semantic routine */
#line 3658 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Integer Word Indexed\n"));
#line 28341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Integer_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3658 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 28360 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 28365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc0
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28384 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28416 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc0;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28426 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28452 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc1
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28475 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28507 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc1;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28517 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28543 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28547 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_set_to_Zero(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_set_to_Zero
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_set_to_Zero
#line 4805 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block set to Zero\n"));
#line 28566 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.1014,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28588 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_set_to_Zero;

    /* semantic routine */
#line 4805 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block set to Zero\n"));
#line 28598 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_set_to_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4805 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block set to Zero\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 28612 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28616 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero
#line 1886 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero\n"));
#line 28635 "icache.c"

  /* validate: 0.32,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x80000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28659 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero;

    /* semantic routine */
#line 1886 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero\n"));
#line 28669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1886 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28688 "icache.c"
    return nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28692 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update
#line 1912 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update\n"));
#line 28711 "icache.c"

  /* validate: 0.33,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x84000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28735 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update;

    /* semantic routine */
#line 1912 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update\n"));
#line 28745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1912 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28765 "icache.c"
    return nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero
#line 1723 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero\n"));
#line 28788 "icache.c"

  /* validate: 0.34,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x88000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28812 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero;

    /* semantic routine */
#line 1723 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero\n"));
#line 28822 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1723 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28841 "icache.c"
    return nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28845 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update
#line 1750 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update\n"));
#line 28864 "icache.c"

  /* validate: 0.35,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x8c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28888 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update;

    /* semantic routine */
#line 1750 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update\n"));
#line 28898 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1750 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28918 "icache.c"
    return nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28922 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Word
#line 2113 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word\n"));
#line 28941 "icache.c"

  /* validate: 0.36,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x90000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28965 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word;

    /* semantic routine */
#line 2113 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word\n"));
#line 28975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2113 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 28994 "icache.c"
    return nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28998 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update
#line 2139 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update\n"));
#line 29017 "icache.c"

  /* validate: 0.37,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x94000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29041 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update;

    /* semantic routine */
#line 2139 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update\n"));
#line 29051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2139 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29071 "icache.c"
    return nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29075 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte
#undef MY_PREFIX
#define MY_PREFIX Store_Byte
#line 2005 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte\n"));
#line 29094 "icache.c"

  /* validate: 0.38,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x98000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29118 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte;

    /* semantic routine */
#line 2005 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte\n"));
#line 29128 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2005 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29147 "icache.c"
    return nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29151 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update
#line 2031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update\n"));
#line 29170 "icache.c"

  /* validate: 0.39,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x9c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29194 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update;

    /* semantic routine */
#line 2031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update\n"));
#line 29204 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2031 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29224 "icache.c"
    return nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29228 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero
#line 1778 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero\n"));
#line 29247 "icache.c"

  /* validate: 0.40,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29271 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero;

    /* semantic routine */
#line 1778 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero\n"));
#line 29281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1778 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29300 "icache.c"
    return nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29304 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update
#line 1804 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update\n"));
#line 29323 "icache.c"

  /* validate: 0.41,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29347 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update;

    /* semantic routine */
#line 1804 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update\n"));
#line 29357 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1804 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29377 "icache.c"
    return nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic
#line 1832 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic\n"));
#line 29400 "icache.c"

  /* validate: 0.42,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29424 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic;

    /* semantic routine */
#line 1832 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic\n"));
#line 29434 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1832 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29453 "icache.c"
    return nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update
#line 1858 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update\n"));
#line 29476 "icache.c"

  /* validate: 0.43,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xac000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29500 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update;

    /* semantic routine */
#line 1858 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update\n"));
#line 29510 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1858 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29530 "icache.c"
    return nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29534 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word
#line 2059 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word\n"));
#line 29553 "icache.c"

  /* validate: 0.44,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29577 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word;

    /* semantic routine */
#line 2059 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word\n"));
#line 29587 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2059 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29606 "icache.c"
    return nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29610 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update
#line 2085 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update\n"));
#line 29629 "icache.c"

  /* validate: 0.45,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29653 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update;

    /* semantic routine */
#line 2085 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update\n"));
#line 29663 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2085 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29683 "icache.c"
    return nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29687 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Multiple_Word(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Load_Multiple_Word
#line 2256 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Multiple Word\n"));
#line 29706 "icache.c"

  /* validate: 0.46,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29730 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Multiple_Word;

    /* semantic routine */
#line 2256 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Multiple Word\n"));
#line 29740 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2256 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      r = RT;
      if (RA >= r)
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      while (r <= 31) {
        GPR(r) = MEM(unsigned, EA, 4);
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29769 "icache.c"
    return nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Multiple_Word(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Multiple_Word
#line 2275 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Multiple Word\n"));
#line 29792 "icache.c"

  /* validate: 0.47,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xbc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29816 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Multiple_Word;

    /* semantic routine */
#line 2275 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Multiple Word\n"));
#line 29826 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2275 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT
          || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      r = RS;
      while (r <= 31) {
        STORE(EA, 4, GPR(r));
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29853 "icache.c"
    return nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single
#line 3466 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single\n"));
#line 29876 "icache.c"

  /* validate: 0.48,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29907 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single;

    /* semantic routine */
#line 3466 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single\n"));
#line 29917 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3466 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29936 "icache.c"
    return nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 29941 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update
#line 3492 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update\n"));
#line 29960 "icache.c"

  /* validate: 0.49,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29991 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3492 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update\n"));
#line 30001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3492 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30021 "icache.c"
    return nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30026 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double
#line 3520 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double\n"));
#line 30045 "icache.c"

  /* validate: 0.50,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30076 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double;

    /* semantic routine */
#line 3520 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double\n"));
#line 30086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3520 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30105 "icache.c"
    return nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30110 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update
#line 3546 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update\n"));
#line 30129 "icache.c"

  /* validate: 0.51,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xcc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30160 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3546 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update\n"));
#line 30170 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3546 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30190 "icache.c"
    return nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single
#line 3579 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single\n"));
#line 30214 "icache.c"

  /* validate: 0.52,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30245 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single;

    /* semantic routine */
#line 3579 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single\n"));
#line 30255 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3579 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30274 "icache.c"
    return nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30279 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update
#line 3605 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update\n"));
#line 30298 "icache.c"

  /* validate: 0.53,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30329 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3605 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update\n"));
#line 30339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3605 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30359 "icache.c"
    return nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30364 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double
#line 3633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double\n"));
#line 30383 "icache.c"

  /* validate: 0.54,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30414 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double;

    /* semantic routine */
#line 3633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double\n"));
#line 30424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3633 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30443 "icache.c"
    return nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30448 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update
#line 3671 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update\n"));
#line 30467 "icache.c"

  /* validate: 0.55,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xdc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30498 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3671 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update\n"));
#line 30508 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3671 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30528 "icache.c"
    return nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc0
#line 3934 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30552 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3934 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30586 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc0;

    /* semantic routine */
#line 3934 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30596 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3934 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30635 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc1
#line 3934 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30660 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3934 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30694 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc1;

    /* semantic routine */
#line 3934 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30704 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3934 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30743 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30749 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc0
#line 3823 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30768 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3823 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30802 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc0;

    /* semantic routine */
#line 3823 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3823 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30844 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30850 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc1
#line 3823 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30869 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3823 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30903 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc1;

    /* semantic routine */
#line 3823 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3823 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30945 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30951 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc0
#line 3771 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 30970 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3771 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31004 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc0;

    /* semantic routine */
#line 3771 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31014 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3771 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31046 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31052 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc1
#line 3771 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 31071 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3771 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31105 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc1;

    /* semantic routine */
#line 3771 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3771 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31147 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc0
#line 4728 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31172 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4728 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31201 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc0;

    /* semantic routine */
#line 4728 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31211 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4728 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31224 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31229 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc1
#line 4728 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31248 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4728 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31277 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc1;

    /* semantic routine */
#line 4728 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31287 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4728 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31300 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc0
#line 4731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31324 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4731 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31353 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc0;

    /* semantic routine */
#line 4731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4731 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31376 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc1
#line 4731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31400 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4731 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31429 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc1;

    /* semantic routine */
#line 4731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31439 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4731 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31452 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc0
#line 3875 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31476 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3875 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31510 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc0;

    /* semantic routine */
#line 3875 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31520 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3875 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31552 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc1
#line 3875 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31577 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3875 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31611 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc1;

    /* semantic routine */
#line 3875 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31621 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3875 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31653 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31659 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc0
#line 4108 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31678 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4108 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31717 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4108 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4108 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31780 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31787 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc1
#line 4108 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31806 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4108 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31845 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4108 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31855 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4108 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31908 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31915 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc0
#line 4014 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 31934 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4014 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31973 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4014 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 31983 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4014 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32036 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32043 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc1
#line 4014 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 32062 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4014 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32101 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4014 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 32111 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4014 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32164 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32171 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc0(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc0
#line 4296 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32190 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4296 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32229 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4296 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4296 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32292 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32299 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc1(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc1
#line 4296 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32318 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4296 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32357 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4296 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32367 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4296 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32420 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc0
#line 4202 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32446 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4202 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32485 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4202 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32495 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4202 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32548 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32555 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc1
#line 4202 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32574 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4202 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32613 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4202 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32623 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4202 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32676 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32683 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Unordered(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Unordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Unordered
#line 4623 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Unordered\n"));
#line 32702 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 32733 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Unordered;

    /* semantic routine */
#line 4623 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Unordered\n"));
#line 32743 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Unordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4623 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0))
        FPSCR_OR_VX(fpscr_vxsnan);
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 32771 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 32776 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc0(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc0
#line 4348 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 32795 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4348 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32824 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc0;

    /* semantic routine */
#line 4348 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 32834 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4348 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33058 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33063 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc1(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc1
#line 4348 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 33082 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4348 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33111 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc1;

    /* semantic routine */
#line 4348 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 33121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4348 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33345 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33350 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc0
#line 4569 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33369 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4569 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33398 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc0;

    /* semantic routine */
#line 4569 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33408 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4569 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33421 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33426 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc1
#line 4569 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33445 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4569 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33474 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc1;

    /* semantic routine */
#line 4569 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33484 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4569 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33497 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0
#line 4576 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33521 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4576 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33550 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0;

    /* semantic routine */
#line 4576 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33560 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4576 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33578 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1
#line 4576 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33602 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4576 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33631 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1;

    /* semantic routine */
#line 4576 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33641 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4576 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33659 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33664 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc0
#line 3901 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33683 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3901 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33717 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc0;

    /* semantic routine */
#line 3901 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3901 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33766 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc1
#line 3901 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33791 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3901 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33825 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc1;

    /* semantic routine */
#line 3901 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33835 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3901 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33874 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33880 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc0
#line 3797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33899 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3797 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33933 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc0;

    /* semantic routine */
#line 3797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 33943 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3797 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33975 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33981 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc1
#line 3797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 34000 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3797 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34034 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc1;

    /* semantic routine */
#line 3797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 34044 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3797 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34076 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34082 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc0(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc0
#line 3745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34101 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34135 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc0;

    /* semantic routine */
#line 3745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34145 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3745 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34177 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34183 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc1(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc1
#line 3745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34202 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34236 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc1;

    /* semantic routine */
#line 3745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34246 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3745 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34284 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc0
#line 4725 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34303 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4725 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34332 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc0;

    /* semantic routine */
#line 4725 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34342 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4725 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34355 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34360 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc1
#line 4725 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34379 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4725 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34408 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc1;

    /* semantic routine */
#line 4725 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34418 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4725 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34431 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34436 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34455 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34494 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34504 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34526 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34552 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34591 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34623 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34630 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34649 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34683 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34725 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34731 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34750 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34784 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34826 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34832 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc0
#line 4734 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34851 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4734 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34880 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc0;

    /* semantic routine */
#line 4734 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34890 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4734 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34903 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34908 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc1
#line 4734 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34927 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4734 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34956 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc1;

    /* semantic routine */
#line 4734 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34966 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4734 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34979 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34984 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35003 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35042 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35052 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35105 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35112 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35131 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35170 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35180 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35233 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35240 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35259 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35298 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35308 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35361 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35368 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35387 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35426 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35436 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35489 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35496 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35515 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35554 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35564 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35617 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35624 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35643 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35682 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35692 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35745 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35752 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35771 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35810 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35820 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35873 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35880 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35899 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35938 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35948 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36001 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36008 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Ordered(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Ordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Ordered
#line 4645 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Ordered\n"));
#line 36027 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 36058 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Ordered;

    /* semantic routine */
#line 4645 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Ordered\n"));
#line 36068 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Ordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4645 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0)
          FPSCR_OR_VX(fpscr_vxvc);
      }
      else if (is_QNaN(*frA, 0) || is_QNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxvc);
      }
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 36102 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 36107 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc0
#line 4708 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36126 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4708 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36149 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc0;

    /* semantic routine */
#line 4708 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36159 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4708 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36182 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc1
#line 4708 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36204 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4708 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36227 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc1;

    /* semantic routine */
#line 4708 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36237 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4708 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36260 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36263 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc0
#line 3713 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36282 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3713 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36311 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc0;

    /* semantic routine */
#line 3713 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3713 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36336 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36341 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc1
#line 3713 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36360 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3713 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36389 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc1;

    /* semantic routine */
#line 3713 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3713 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36414 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36419 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36438 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36477 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36487 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36509 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36535 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36574 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36584 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36606 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36613 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36632 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36666 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36676 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36708 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36714 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36733 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36809 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36815 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36834 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36873 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 36883 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36936 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36943 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36962 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37001 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 37011 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37064 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37071 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37090 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37129 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37139 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37192 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37199 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37218 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37257 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37267 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37320 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37327 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37346 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37385 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37395 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37448 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37455 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37474 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37513 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37523 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37576 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37602 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37641 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37651 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37704 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37711 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37730 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37769 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37779 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37832 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37839 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_FPSCR(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_FPSCR
#line 4679 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from FPSCR\n"));
#line 37858 "icache.c"

  /* validate: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0xfc000080)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4679 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.X.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 37881 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_FPSCR;

    /* semantic routine */
#line 4679 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from FPSCR\n"));
#line 37891 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4679 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned field = FPSCR_FIELD(BFA);
      CR_SET(BF, field);
      FPSCR_SET(BFA, 0); /* FPSCR_END fixes up FEX/VX */
      FPSCR_END(0);
    }
#line 37908 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #undef get-values-from-icache|put-values-in-icache */
#line 37911 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc0
#line 4702 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37930 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4702 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37953 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc0;

    /* semantic routine */
#line 4702 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 37963 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4702 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 37979 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 37982 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc1
#line 4702 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 38001 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4702 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38024 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc1;

    /* semantic routine */
#line 4702 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 38034 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4702 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 38050 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 38053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc0
#line 3704 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38072 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3704 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38101 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc0;

    /* semantic routine */
#line 3704 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38111 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3704 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38126 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38131 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc1
#line 3704 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38150 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3704 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38179 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc1;

    /* semantic routine */
#line 3704 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38189 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3704 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38204 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38209 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38228 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38267 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38277 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38299 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38306 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38325 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38364 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38374 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38396 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38403 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38422 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38456 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38466 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38498 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38523 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38557 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38599 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38605 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38624 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38663 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38726 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38752 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38791 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38801 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38854 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 38880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 38929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38982 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38989 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 39008 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39047 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 39057 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39110 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39136 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39175 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39185 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39238 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39245 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39264 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39303 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39313 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39366 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39392 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39431 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39494 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39501 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39520 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39559 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39569 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39622 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39687 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39697 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39719 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39726 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39745 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39784 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39816 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39823 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39842 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39876 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39886 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 39918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 39924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39943 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39977 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39987 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 40019 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 40025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40044 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40083 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40146 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40172 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40274 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40281 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40300 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40339 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40349 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40402 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40409 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40428 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40467 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40477 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40530 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40537 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40556 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40595 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40665 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40684 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40723 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40733 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40786 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40793 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40812 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40851 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41042 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc0
#line 4686 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41068 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41093 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc0;

    /* semantic routine */
#line 4686 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41103 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4686 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41118 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc1
#line 4686 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41140 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41165 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc1;

    /* semantic routine */
#line 4686 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41175 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4686 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41190 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41193 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc0
#line 3731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41212 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3731 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41241 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc0;

    /* semantic routine */
#line 3731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41251 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3731 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41266 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc1
#line 3731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41290 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3731 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41319 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc1;

    /* semantic routine */
#line 3731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41329 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3731 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41344 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41368 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41407 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41417 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41439 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41446 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41465 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41504 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41514 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41536 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41543 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41562 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41596 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41606 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41638 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41644 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41663 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41697 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41707 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41739 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41745 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41764 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41803 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41866 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41873 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41892 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41931 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41941 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41994 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42148 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42187 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42250 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42276 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42315 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42404 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42443 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42453 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42506 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42532 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42571 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42581 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42660 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42699 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42859 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42866 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42885 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42924 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42934 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42956 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42963 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 42982 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43016 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43026 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43058 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43064 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 43083 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43117 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43127 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43159 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43165 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43184 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43223 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43233 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43286 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43293 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43312 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43351 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43361 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43414 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43421 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43440 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43479 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43489 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43542 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43549 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43568 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43607 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43670 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43696 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43735 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43798 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43805 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43824 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43863 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43873 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43926 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 43952 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43991 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 44080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44182 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44189 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44208 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44247 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44257 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44279 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44286 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44305 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44344 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44354 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44376 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44383 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44402 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44436 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44446 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44478 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44484 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44503 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44537 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44547 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44579 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44585 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44604 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44643 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44653 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44706 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44713 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44732 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44771 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44781 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44834 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44860 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44899 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 44909 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44962 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44969 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44988 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45027 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 45037 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45090 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45097 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45116 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45155 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45165 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45218 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45244 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45283 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45346 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45353 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45372 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45411 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45481 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45500 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45539 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45549 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45602 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45628 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45667 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45677 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45699 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45706 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45725 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45764 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45774 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45796 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45803 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45822 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45856 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45866 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45898 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 45904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45923 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45957 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45967 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45999 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 46005 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46024 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46063 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46073 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46126 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46133 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46152 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46191 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46201 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46254 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46261 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46280 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46319 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46329 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46382 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46389 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46408 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46447 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46457 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46510 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46638 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46645 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46664 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46703 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46713 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46766 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46792 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46831 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46841 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46920 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46959 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47022 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47029 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc0
#line 3722 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47048 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3722 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47077 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc0;

    /* semantic routine */
#line 3722 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47087 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3722 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47102 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47107 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc1
#line 3722 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47126 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3722 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47155 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc1;

    /* semantic routine */
#line 3722 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47165 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3722 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47180 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47275 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47282 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47301 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47340 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47350 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47372 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47379 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47398 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47432 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47442 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47480 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47499 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47533 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47543 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47575 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47600 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47639 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47649 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47702 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47728 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47830 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47856 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47895 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 47905 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47958 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47984 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48023 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 48033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48086 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48093 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48112 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48151 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48214 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48221 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48240 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48279 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48342 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48368 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48407 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48417 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48470 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48477 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48496 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48535 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48605 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48624 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48663 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48695 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48702 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48721 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48760 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48770 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48792 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48799 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48818 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48852 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48862 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48900 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48919 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48953 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48963 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48995 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 49001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49148 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49187 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49250 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49276 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49315 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49404 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49443 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49453 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49506 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49532 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49571 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49581 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49660 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49699 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49890 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49897 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49916 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49955 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49965 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50044 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50083 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50115 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50122 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50141 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50180 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50190 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50212 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50219 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50238 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50272 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50282 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50314 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50320 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50339 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50373 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50383 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50415 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50421 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50440 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50479 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50489 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50542 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50549 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50568 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50607 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50670 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50696 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50735 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50798 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50805 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50824 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50863 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50873 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50926 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 50952 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50991 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 51001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 51080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 51129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51182 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51189 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51208 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51247 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51257 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51310 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51317 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51336 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51375 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51385 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51464 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51503 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51535 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51542 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51561 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51600 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51610 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51632 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51639 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51658 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51692 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51702 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51734 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51759 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51793 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51803 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51835 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51860 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51899 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 51909 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51962 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51969 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51988 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52027 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 52037 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52090 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52097 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52116 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52155 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52165 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52218 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52244 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52283 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52346 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52353 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52372 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52411 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52481 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52500 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52539 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52549 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52602 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52628 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52667 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52677 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52730 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52737 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52756 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52795 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52805 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52858 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52865 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52884 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52923 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 52933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 52955 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52962 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52981 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53020 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 53030 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 53052 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53059 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53078 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53112 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53122 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53154 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53179 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53213 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53223 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53255 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53261 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53280 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53319 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53329 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53382 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53389 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53408 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53447 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53457 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53510 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53638 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53645 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53664 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53703 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53713 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53766 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53792 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53831 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53841 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53920 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53959 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54022 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54029 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54048 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54087 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54150 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54157 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54176 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54215 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54225 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54304 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54343 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54375 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54382 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54401 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54440 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54450 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54472 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54479 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54498 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54532 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54574 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54599 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54633 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54643 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54675 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54681 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54700 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54739 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54749 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54802 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54809 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54828 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54867 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54877 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54930 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54937 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 54956 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54995 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55005 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55058 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55065 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 55084 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55123 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55133 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55186 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55193 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55212 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55251 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55261 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55314 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55321 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55340 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55379 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55389 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55442 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55449 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55468 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55507 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55517 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55570 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55596 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55635 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55698 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55705 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55724 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55795 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55821 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55860 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55870 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55892 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55899 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 55918 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55952 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 55962 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 55994 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 56000 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 56019 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56053 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 56063 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 56095 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 56101 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56120 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56159 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56169 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56222 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56229 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56248 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56287 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56297 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56350 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56357 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56376 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56415 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56425 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56478 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56485 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56504 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56543 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56553 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56606 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56613 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56632 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56671 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56681 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56734 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56760 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56799 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56862 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56869 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 56888 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56927 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 56937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56990 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56997 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 57016 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57055 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 57065 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57215 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57222 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57241 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57280 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57290 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57312 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57319 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57338 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57372 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57382 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57414 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57439 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57473 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57515 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57521 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57540 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57579 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57589 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57642 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57649 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57668 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57707 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57717 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57770 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57777 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57796 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57835 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57845 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57898 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57905 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57924 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57963 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57973 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58026 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58033 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58052 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58091 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58101 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58154 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58180 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58219 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58229 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58282 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58289 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58308 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58347 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58357 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58410 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58436 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58475 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58538 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58545 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58564 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58603 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58613 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58635 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58642 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58661 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58700 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58732 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58739 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58758 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58792 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58802 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58834 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58840 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58859 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58893 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58903 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58935 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58941 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 58960 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58999 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59009 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59062 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59069 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 59088 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59127 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59190 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59197 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59216 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59255 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59265 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59318 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59325 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59344 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59383 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59393 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59446 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59453 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59472 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59511 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59521 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59574 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59600 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59639 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59649 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59702 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59728 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59830 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59856 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59895 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59905 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59958 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 59984 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60023 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60055 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60062 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 60081 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60120 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60130 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60152 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60159 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60178 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60212 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60222 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60254 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60260 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60279 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60313 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60355 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60361 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60380 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60419 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60429 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60482 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60489 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60508 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60547 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60557 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60610 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60617 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60636 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60675 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60685 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60738 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60745 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60764 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60803 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60866 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60873 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 60892 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60931 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 60941 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60994 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 61020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 61069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61148 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61187 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61250 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61276 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61315 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc0
#line 4674 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61404 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4674 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61428 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc0;

    /* semantic routine */
#line 4674 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61438 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4674 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61453 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc1
#line 4674 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61476 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4674 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61500 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc1;

    /* semantic routine */
#line 4674 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61510 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4674 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61525 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61548 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61587 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61619 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61626 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61645 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61684 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61694 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61716 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61723 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61742 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61776 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61824 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61843 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61877 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61887 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61919 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61925 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 61944 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61983 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 61993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62046 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 62072 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 62121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62174 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62302 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62328 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62367 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62584 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62623 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62633 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62686 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62693 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62712 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62751 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62761 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62814 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62821 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62840 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62879 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62942 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62949 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 62968 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63007 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63039 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63046 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 63065 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63104 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63114 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63136 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63143 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63162 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63196 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63206 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63238 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63244 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63263 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63297 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63307 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63339 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63364 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63403 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63413 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63466 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63492 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63531 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63594 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63601 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63620 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63659 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63722 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63748 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63787 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63797 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63850 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 63876 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63915 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 63925 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63978 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63985 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 64004 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64043 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 64053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64106 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64132 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64171 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64181 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64234 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64241 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64260 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64299 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64309 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64362 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64369 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64388 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64427 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64437 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64459 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64466 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64485 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64524 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64534 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64556 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64563 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64582 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64616 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64626 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64664 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64683 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64717 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64759 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64784 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64823 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64886 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64893 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64912 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64951 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64961 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65142 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65168 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65207 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65270 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65296 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65335 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65345 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65526 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65552 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65591 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65680 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65719 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65782 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65789 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65808 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65847 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65857 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65879 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65886 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65905 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65944 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65954 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65976 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65983 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 66002 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66036 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66046 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66084 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 66103 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66137 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66147 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66179 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66306 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66332 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66434 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66441 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66460 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66499 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66569 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66588 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66627 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66637 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66690 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66697 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66716 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66755 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66765 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66844 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66883 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66946 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66953 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 66972 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67011 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 67100 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67139 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67149 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67202 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67209 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc0
#line 4691 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67228 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4691 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4691 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67254 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc0;

    /* semantic routine */
#line 4691 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67264 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4691 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67285 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67289 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc1
#line 4691 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67308 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4691 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4691 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67334 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc1;

    /* semantic routine */
#line 4691 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67344 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4691 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67365 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67369 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67388 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67427 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67437 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67459 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67466 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67485 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67524 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67534 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67556 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67563 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67582 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67616 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67626 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67664 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67683 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67717 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67759 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67784 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67823 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67886 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67893 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67912 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67951 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67961 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68142 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68168 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68207 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68270 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68296 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68335 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68345 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68526 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68552 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68591 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68680 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68719 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68782 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68789 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68808 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68847 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68857 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68879 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68886 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68905 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68944 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68954 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68976 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68983 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 69002 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69036 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69046 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69084 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 69103 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69137 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69147 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69179 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69306 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69332 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69434 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69441 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69460 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69499 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69569 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69588 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69627 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69637 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69690 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69697 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69716 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69755 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69765 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69844 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69883 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69946 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69953 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 69972 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70011 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 70100 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70139 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70149 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70202 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70209 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70228 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70267 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70277 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70299 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70306 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70325 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70364 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70374 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70396 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70403 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70422 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70456 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70466 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70498 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70523 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70557 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70599 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70605 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70624 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70663 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70726 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70752 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70791 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70801 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70854 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 70880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 70929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70982 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70989 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 71008 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71047 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 71057 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71110 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71136 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71175 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71185 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71238 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71245 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71264 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71303 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71313 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71366 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71392 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71431 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71494 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71501 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71520 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71559 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71569 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71622 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71687 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71697 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71719 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71726 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71745 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71784 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71816 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71823 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71842 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71876 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71886 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 71918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 71924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71943 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71977 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71987 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 72019 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 72025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72044 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72083 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72146 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72172 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72274 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72281 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72300 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72339 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72349 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72402 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72409 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72428 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72467 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72477 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72530 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72537 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72556 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72595 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72665 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72684 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72723 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72733 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72786 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72793 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72812 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72851 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73042 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73068 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73107 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73139 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73146 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73165 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73204 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73214 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73236 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73243 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73262 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73296 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73306 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73338 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73344 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73363 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73397 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73407 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73439 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73464 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73503 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73566 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73573 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73592 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73631 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73641 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73694 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73720 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73759 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73822 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73829 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73848 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73887 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73897 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73950 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73957 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 73976 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74015 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74025 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74085 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 74104 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74143 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74153 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74206 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74213 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74232 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74334 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74341 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74360 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74399 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74409 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74462 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74469 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74488 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74527 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74537 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74559 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74566 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74585 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74624 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74634 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74656 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74663 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74682 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74716 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74726 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74783 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74817 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74827 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74859 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74865 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 74884 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74923 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 74933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74986 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74993 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 75012 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75051 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 75061 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75114 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75140 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75179 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75189 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75242 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75249 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75268 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75307 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75370 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75377 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75396 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75435 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75498 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75505 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75524 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75563 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75573 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75626 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75633 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75652 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75691 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75701 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75754 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75761 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75780 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75819 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75882 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 75908 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75947 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 75957 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 75979 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75986 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 76005 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76044 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 76054 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 76076 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76102 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76136 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76146 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76203 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76237 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76247 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76279 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76304 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76343 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76406 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76413 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76432 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76471 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76534 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76541 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76560 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76599 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76662 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76669 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76688 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76727 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76737 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76790 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76797 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76816 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76855 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76865 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76925 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76944 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76983 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77046 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77072 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77174 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77302 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77328 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77367 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77399 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77406 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77425 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77464 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77474 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77496 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77522 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77556 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77566 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77604 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77623 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77657 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77667 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77699 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77705 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77724 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77826 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77833 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77852 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77891 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77901 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 77980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78082 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78089 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 78108 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78147 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78210 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78217 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78236 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78275 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78285 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78338 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78364 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78403 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78413 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78466 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78492 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78531 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78594 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78601 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78620 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78659 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78722 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78748 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78787 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78797 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78819 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78826 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78845 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78884 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78894 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78916 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78923 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 78942 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78976 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 78986 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 79043 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79077 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 79087 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79119 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79246 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79272 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79311 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79400 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79439 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79502 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79528 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79567 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79577 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79630 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79656 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79695 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79705 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79784 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79823 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79886 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79893 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 79912 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79951 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 79961 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 80040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 80089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80142 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc0
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80168 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80207 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc0;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80239 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80246 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc1
#line 4745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80265 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80304 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc1;

    /* semantic routine */
#line 4745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80314 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4745 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80336 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80343 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80362 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80396 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80406 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80444 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80463 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80497 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80507 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80539 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80545 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc0
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80564 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80603 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80613 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80666 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80673 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc1
#line 4061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80692 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80731 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4061 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80794 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80820 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80859 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80922 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80929 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80948 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80987 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81050 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81057 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc0
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81076 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 81115 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81125 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc1
#line 4249 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4249 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 81243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4249 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4249 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81306 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc0
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81332 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 81371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81434 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81441 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc1
#line 4155 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81460 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4155 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 81499 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 4155 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4155 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; unsigned64 u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81569 "icache.c"

#undef insn
  }
}

#endif /* _ICACHE_C_*/
